@@ -4,8 +4,6 @@ use v5.26;
 
 package ChordPro;
 
-use App::Packager;
-
 use ChordPro::Utils;
 use ChordPro::Chords;
 use ChordPro::Output::Common;
@@ -75,6 +73,18 @@ our $config;
 
 package ChordPro;
 
+use ChordPro::Paths;
+use Encode qw(decode decode_utf8 encode_utf8);
+
+sub import {
+    # Add private library.
+    my $lib = CP->privlib;
+    for ( @INC ) {
+	return if $_ eq $lib;
+    }
+    unshift( @INC, $lib );
+}
+
 sub ::run {
     binmode(STDERR, ':utf8');
     binmode(STDOUT, ':utf8');
@@ -138,9 +148,7 @@ sub chordpro {
 	    # No default if more than one input document.
 	    die("Please use \"--output\" to specify the output file name\n");
 	}
-	my $f = $ARGV[0];
-	$f =~ s/\.\w+$/.pdf/;
-	$f .= ".pdf" if $f eq $ARGV[0];
+	my $f = CP->sibling( $ARGV[0], ext => ".pdf" );
 	$options->{output} = $f;
 	warn("Writing output to $f\n") if $options->{verbose};
     }
@@ -196,6 +204,7 @@ sub chordpro {
 		$opts->{meta}->{__config} = \@cfg;
 	    }
 	}
+	$opts->{generate} = $options->{generate};
 	# Wx runs on temp files, so pass real filename in.
 	$opts->{filesource} = $options->{filesource};
 	$s->parse_file( $file, $opts );
@@ -276,19 +285,8 @@ sub chordpro {
 }
 
 sub ::dump {
-    use Data::Dumper qw();
-    local $Data::Dumper::Sortkeys  = 1;
-    local $Data::Dumper::Indent    = 1;
-    local $Data::Dumper::Quotekeys = 0;
-    local $Data::Dumper::Deparse   = 1;
-    local $Data::Dumper::Terse     = 1;
-    local $Data::Dumper::Trailingcomma = 1;
-    local $Data::Dumper::Useperl = 1;
-    local $Data::Dumper::Useqq     = 0; # I want unicode visible
-
-    my $s = Data::Dumper::Dumper @_;
-    defined wantarray or warn $s;
-    return $s;
+    use ChordPro::Dumper;
+    ddp(@_);
 }
 
 ################ Options and Configuration ################
@@ -706,37 +704,7 @@ sub app_setup {
     }
 
     # Config files.
-    if ( -d "/etc" ) {          # some *ux
-        $configs{sysconfig} =
-          File::Spec->catfile( "/", "etc", "$app_lc.json" );
-    }
-
-    if ( $ENV{XDG_CONFIG_HOME} && -d $ENV{XDG_CONFIG_HOME} ) {
-	$configs{userconfig} =
-	  File::Spec->catfile( $ENV{XDG_CONFIG_HOME}, $app_lc, "$app_lc.json" );
-	$ENV{CHORDPRO_LIB} ||= File::Spec->catfile( $ENV{XDG_CONFIG_HOME}, $app_lc);
-    }
-    elsif ( $ENV{HOME} && -d $ENV{HOME} ) {
-	my $dir = File::Spec->catfile( $ENV{HOME}, ".config" );
-        if ( -d $dir ) {
-            $configs{userconfig} =
-              File::Spec->catfile( $dir, $app_lc, "$app_lc.json" );
-	    $ENV{CHORDPRO_LIB} ||= File::Spec->catfile( $dir, $app_lc );
-        }
-        else {
-	    $dir = File::Spec->catfile( $ENV{HOME}, ".$app_lc" );
-            $configs{userconfig} =
-              File::Spec->catfile( $dir, "$app_lc.json" );
-	    $ENV{CHORDPRO_LIB} ||= $dir;
-        }
-    }
-
-    if ( -s ".$app_lc.json" ) {
-        $configs{config} = ".$app_lc.json";
-    }
-    else {
-        $configs{config} = "$app_lc.json";
-    }
+    %configs = %{ CP->configs };
 
     my $options =
       {
@@ -902,8 +870,8 @@ sub app_setup {
         # Load Pod::Usage only if needed.
         require Pod::Usage;
         Pod::Usage->import;
-	my $f = $manual == 2 ? "pod/Config.pod" : "pod/ChordPro.pod";
-        unshift( @_, -input => getresource($f) );
+	my $f = $manual == 2 ? "Config.pod" : "ChordPro.pod";
+        unshift( @_, -input => CP->findres( $f, class => "pod" ) );
         &pod2usage;
     };
 
@@ -935,11 +903,13 @@ sub app_setup {
         for ( $clo->{$config} ) {
             if ( defined($_) ) {
                 foreach my $c ( @$_ ) {
+		    my $try = $c;
 		    # Check for resource names.
-		    if ( $c !~ m;[/.]; ) {
-			$c = ::rsc_or_file( $c, "config" );
+		    if ( ! -r $try ) {
+			$try = CP->findcfg($c);
 		    }
-                    die("$c: $!\n") unless -r $c;
+                    die("$c: $!\n") unless $try && -r $try;
+		    $c = $try;
                 }
                 next;
             }
@@ -1049,19 +1019,17 @@ EndOfAbout
     exit $exit if defined $exit;
 }
 
-use Cwd qw(realpath);
+use List::Util qw(uniq);
 
 sub ::runtimeinfo {
     my $level = shift // "normal";
-
+    my $cp = ChordPro::Paths->get;
     my $fmt   = "  %-22.22s %-10s\n";
-    my $fmtv  = defined($Wx::VERSION) ? "  %s version %-10s\n" : $fmt;
-    my $fmtvv = defined($Wx::VERSION) ? "  %s %-10s\n" : $fmt;
 
     # Sometimes version numbers are localized...
     my $dd = sub { my $v = $_[0]; $v =~ s/,/./g; $v };
 
-    my $msg = sprintf( $fmtv, "ChordPro core", $dd->($VERSION) );
+    my $msg = sprintf( $fmt, "ChordPro core", $dd->($VERSION) );
     $msg =~ s/core/reference/ if $::options->{reference};
     if ( $VERSION =~ /_/ ) {
 	$msg =~ s/\n$/ (Unsupported development snapshot)\n/;
@@ -1074,31 +1042,55 @@ sub ::runtimeinfo {
 	return $msg;
     }
 
-    $msg .= sprintf( $fmtv, "Perl", $^V );
-    $msg =~ s/\n$/ ($^X)\n/;
-    if ( $App::Packager::PACKAGED ) {
-	my $p = App::Packager::Packager();
-	$p .= " Packager" unless $p =~ /packager/i;
-	$msg .= sprintf( $fmtv, $p, $dd->(App::Packager::Version()) );
-    }
+    $msg .= sprintf( $fmt, "Perl", $^V );
+    $msg =~ s/\n$/sprintf(" (%s)\n", $cp->display($^X))/e;
 
+    if ( CP->packager ) {
+	$msg .= sprintf( $fmt, CP->packager." Packager", CP->packager_version );
+    }
+ 
     # Determine resource path.
     my @p;
-    if ( $ENV{CHORDPRO_LIB} ) {
-	$msg .= sprintf( $fmtvv, "CHORDPRO_LIB", $ENV{CHORDPRO_LIB} );
-	@p = splitpath($ENV{CHORDPRO_LIB});
+    my $tag = "CHORDPRO_LIB";
+    if ( defined $ENV{CHORDPRO_LIB} ) {
+	for ( $cp->path($ENV{CHORDPRO_LIB}) ) {
+	    $msg .= sprintf( $fmt, $tag, $cp->display($_) );
+	    $tag = "";
+	    push( @p, $cp->display($_) );
+	}
     }
-    push( @p, realpath( App::Packager::GetResourcePath() ) );
-    my $tag = "Resource path";
-    for ( @p ) {
-	$msg .= sprintf( $fmtvv, $tag, $_ );
+    for ( qw( XDG_CONFIG_HOME ) ) {
+	if ( defined($ENV{$_}) && $ENV{$_} ne "" ) {
+	    $msg .= sprintf( $fmt, $_, $cp->display($ENV{$_}) );
+	}
+    }
+    push( @p,  map { $cp->display($_) } @{ $cp->resdirs } );
+    $tag = "Resource path";
+    for ( uniq(@p) ) {
+	$msg .= sprintf( $fmt, $tag, $_ );
 	$tag = "";
     }
+    eval { require ChordPro::Delegate::ABC;
+	   my $x;
+	   if ( ChordPro::Delegate::ABC::have_xs() ) {
+	       $x = ChordPro::Delegate::ABC::packaged_qjs();
+	       $msg .= sprintf( $fmt, "ABC support",
+				$x->[0] . " (" . $x->[1] . ")" );
+	   }
+	   elsif ( $x = findexe( "abc2svg", "silent" )
+		        || findexe( "abcnode", "silent" ) ) {
+	       $msg .= sprintf( $fmt, "ABC support", $cp->display($x) );
+	   }
+	   elsif ( $x = ChordPro::Delegate::ABC::packaged_qjs() ) {
+	       $msg .= sprintf( $fmt, "ABC support",
+				$cp->display($x->[0]) . " (" . $cp->display($x->[-1]) . ")" );
+	   }
+    };
 
     my $vv = sub {
 	my ( $mod ) = @_;
 	no strict 'refs';
-	$msg .= sprintf( $fmtv, $mod, $dd->(${${"${mod}::"}{VERSION}}) );
+	$msg .= sprintf( $fmt, $mod, $dd->(${${"${mod}::"}{VERSION}}) );
 	return unless $level eq "extensive";
 	chomp($msg);
 	my $pm = $mod =~ s;::;/;gr . ".pm";
@@ -1117,8 +1109,8 @@ sub ::runtimeinfo {
     $msg .= "\nModules and libraries:\n";
     if ( defined $Wx::VERSION ) {
 	no strict 'subs';
-	$msg .= sprintf( $fmtv, "wxPerl", $dd->($Wx::VERSION) );
-	$msg .= sprintf( $fmtv, "wxWidgets", $dd->(Wx::wxVERSION) );
+	$msg .= sprintf( $fmt, "wxPerl", $dd->($Wx::VERSION) );
+	$msg .= sprintf( $fmt, "wxWidgets", $dd->(Wx::wxVERSION) );
     }
 
     local $SIG{__WARN__} = sub {};
@@ -1130,7 +1122,7 @@ sub ::runtimeinfo {
     };
     eval { require HarfBuzz::Shaper;
 	$vv->("HarfBuzz::Shaper");
-	$msg .= sprintf( $fmtv, "HarfBuzz library", $dd->(HarfBuzz::Shaper::hb_version_string()) );
+	$msg .= sprintf( $fmt, "HarfBuzz library", $dd->(HarfBuzz::Shaper::hb_version_string()) );
     };
     $vv->("File::LoadLines");
     eval { require PDF::Builder;
@@ -1138,6 +1130,9 @@ sub ::runtimeinfo {
     };
     eval { require PDF::API2;
 	$vv->("PDF::API2");
+	eval { require PDF::API2::XS;
+	       $vv->("PDF::API2::XS");
+	};
     };
     eval { require SVGPDF;
 	$vv->("SVGPDF");
@@ -1145,6 +1140,9 @@ sub ::runtimeinfo {
     eval { require Font::TTF;
 	$vv->("Font::TTF");
     };
+    eval { require JavaScript::QuickJS;
+	$vv->("JavaScript::QuickJS");
+    };
     return $msg;
 }
 
@@ -1240,40 +1238,6 @@ EndOfUsage
     exit $exit if defined $exit;
 }
 
-################ Resources ################
-
-use Encode qw(decode decode_utf8 encode_utf8);
-
-sub ::rsc_or_file {
-    my ( $c, $cfg ) = @_;
-    my $f = $c;
-    $cfg .= "/" if $cfg;
-
-    # Check for resource names.
-    if ( $f !~ m;[/.]; ) {
-	if ( $c =~ /^(.+):(.*)/ ) {
-	    $f = $cfg . lc($1) . "/" . lc($2) . ".json";
-	}
-	else {
-	    $f = $cfg . lc($c) . ".json";
-	}
-    }
-    if ( $ENV{CHORDPRO_LIB} ) {
-	my @libs = splitpath($ENV{CHORDPRO_LIB});
-	foreach my $lib ( @libs ) {
-	    $lib = expand_tilde($lib);
-	    warn("RSC1: $lib/$f\n") if $options->{debug};
-	    return $lib . "/" . $f if -r $lib . "/" . $f;
-	}
-    }
-
-    warn("RSC3: $f\n") if $options->{debug};
-    my $t = getresource($f);
-    return defined($t) ? $t : $c;
-}
-
-use lib ( grep { defined } getresource("CPAN") );
-
 =head1 FONTS
 
 There are two ways to specify fonts: with a font filename, and a
